import 'package:flutter/material.dart';

/// A custom implementation of AsyncSnapshot that also contains the progress of
/// the async operation. This is useful for showing a loading indicator with
/// exact progress information.
class AsyncSnapshotWithProgress<T, ProgressType extends Object>
    implements AsyncSnapshot<T> {
  const AsyncSnapshotWithProgress._(
    this.connectionState,
    this.data,
    this.error,
    this.stackTrace,
    this.progress,
  )   : assert(!(data != null && error != null)),
        assert(stackTrace == null || error != null);

  /// Creates an [AsyncSnapshotWithProgress] in [ConnectionState.none] with null
  /// data and error.
  const AsyncSnapshotWithProgress.nothing()
      : this._(ConnectionState.none, null, null, null, null);

  /// Creates an [AsyncSnapshotWithProgress] in [ConnectionState.waiting] with
  /// null data and error.
  const AsyncSnapshotWithProgress.waiting(ProgressType progress)
      : this._(ConnectionState.waiting, null, null, null, progress);

  /// Creates an [AsyncSnapshotWithProgress] in the specified [state] and with
  /// the specified [data].
  const AsyncSnapshotWithProgress.withData(ConnectionState state, T data)
      : this._(state, data, null, null, null);

  /// Creates an [AsyncSnapshotWithProgress] in the specified [state] with the
  /// specified [error] and a [stackTrace].
  ///
  /// If no [stackTrace] is explicitly specified, [StackTrace.empty] will be
  /// used instead.
  const AsyncSnapshotWithProgress.withError(
    ConnectionState state,
    Object error, [
    StackTrace stackTrace = StackTrace.empty,
    ProgressType? progress,
  ]) : this._(state, null, error, stackTrace, progress);

  @override
  final ConnectionState connectionState;

  @override
  final T? data;

  @override
  final StackTrace? stackTrace;

  final ProgressType? progress;

  @override
  final Object? error;

  @override
  bool get hasData => data != null;

  @override
  bool get hasError => error != null;

  @override
  AsyncSnapshotWithProgress<T, ProgressType> inState(ConnectionState state) =>
      AsyncSnapshotWithProgress<T, ProgressType>._(
        state,
        data,
        error,
        stackTrace,
        progress,
      );

  @override
  T get requireData {
    if (hasData) return data!;
    // ignore: only_throw_errors
    if (hasError) throw error!;
    throw StateError('Snapshot has neither data nor error');
  }
}
